from collections import defaultdict
from xml.etree import ElementTree

import requests
from requests.auth import HTTPDigestAuth

from spread_core.tools.base_interface import AbstractInterfaceClass
from spread_core.errors.project_errors import ProjectError


def etree_to_dict(t):
    d = {t.tag: {} if t.attrib else None}
    children = list(t)
    if children:
        dd = defaultdict(list)
        for dc in map(etree_to_dict, children):
            for k, v in dc.items():
                dd[k].append(v)
        d = {t.tag: {k: v[0] if len(v) == 1 else v for k, v in dd.items()}}
    if t.attrib:
        d[t.tag].update((k, v) for k, v in t.attrib.items())
    if t.text:
        text = t.text.strip()
        if children or t.attrib:
            if text:
              d[t.tag]['#text'] = text
        else:
            d[t.tag] = text
    return d


class AnimeoAdapter(AbstractInterfaceClass):
    AUTH_PART = 'xml'

    def __init__(self):
        super(AnimeoAdapter, self).__init__(None)
        self._url = None
        self._auth = None
        self._session = None

    def set_bus_params(self, bus: dict):
        try:
            self._url = f'{bus["url"]}/{self.AUTH_PART}'
            self._auth = (bus['login'], bus['password'])
        except (TypeError, KeyError):
            raise ProjectError(f'Некорректная конфигурация шины')

    def set_event_handler(self, _): pass
    def set_external_cmd_handler(self, _): pass
    def set_status_handler(self, _): pass

    def start(self):
        pass

    @property
    def session(self):
        if self._session is None:
            self._session = requests.session()
            self._session.auth = HTTPDigestAuth(*self._auth)

        return self._session

    def _make_request(self, url):
        response = self.session.get(f'{self._url}/{url}')
        if response.history and response.history[0].status_code == 401:
            self.session.headers['Authorization'] = response.request.headers.get('Authorization')

        return response

    def send(self, url_segment: str, **kwargs):
        url_params = '&'.join([f'{key}={value}' for key, value in kwargs.items()])
        response = self._make_request(f'{url_segment}?{url_params}')

        print('->', response.url)
        try:
            response.raise_for_status()
        except requests.HTTPError as ex:
            print(' <-', ex)
            raise ex
        else:
            root = ElementTree.XML(response.content.decode())
            tree = etree_to_dict(root)
            print(' <-', tree)
            return tree
