from spread_core.errors.project_errors import ProjectError
from spread_dali.address import Short, Broadcast, Group, BroadcastUnaddressed, Address

BINDING_KEY = 'binding'
DEVICE_KEY = 'device'
GROUP_KEY = 'group'

DEVICE = 'Device'
GROUP = 'Group'
BROADCAST = 'Broadcast'
UNADDRESSED = 'Unaddressed'


def of(data) -> Address:
    try:
        binding_type = data[BINDING_KEY]

        if binding_type == DEVICE:
            return Short(data[DEVICE_KEY])

        elif binding_type == GROUP:
            return Group(data[GROUP_KEY])

        elif binding_type == BROADCAST:
            return Broadcast()

        elif binding_type == UNADDRESSED:
            return BroadcastUnaddressed()

        else:
            raise BindingError.unknown_binding(binding_type)

    except KeyError as ex:
        raise BindingError.missing_key(ex)
    except ValueError as ex:
        raise BindingError.invalid_value(str(ex))


class BindingError(Exception):
    MISSING_KEY = 1
    INVALID_BINDING = 2
    INVALID_VALUE = 3

    def __init__(self, code, *args):
        super(BindingError, self).__init__(*args)
        self.code = code

    @classmethod
    def unknown_binding(cls, binding_type):
        return cls(cls.INVALID_BINDING, 'Unknown binding "%s"' % binding_type)

    @classmethod
    def missing_key(cls, key):
        return cls(cls.MISSING_KEY, 'Missing key %s' % key)

    @classmethod
    def invalid_value(cls, desc):
        return cls(cls.INVALID_VALUE, desc)
