from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from .enginery import Enginery, _EngineryFunit


class _Climate(Enginery):
    def on_state(self, topic: State, variable: Variable):
        super(_Climate, self).on_state(topic, variable)

        for subginery in self._subgineries:
            subginery.on_level_changed(self, topic.entity_address.funit_type)


class TemperatureSensor(_Climate):
    pass


class Thermoregulator(_Climate):
    pass


class Fan(_Climate):
    pass


class HeatedFloor(_Climate):
    pass


class _ClimateFunit(_EngineryFunit):
    entity_type = _Climate


_Climate.funit_class = _ClimateFunit


class Temperature(_ClimateFunit):
    entity_type = TemperatureSensor

    def set(self):
        raise self.NotImplemented()


class On(_ClimateFunit):
    value_type = bool


class TargetTemperature(_ClimateFunit):
    value_type = int
    entity_type = Thermoregulator


class Preset(_ClimateFunit):
    entity_type = Thermoregulator

    MANUAL = 'Manual'
    AUTO = 'Auto'
    PROTECTION = 'Protection'
    DAY = 'Day'
    NIGHT = 'Night'
    STANDBY = 'Standby'
    COMFORT = 'Comfort'
    TIMER = 'Timer'

    variants = [MANUAL, AUTO, PROTECTION, DAY, NIGHT, STANDBY, COMFORT, TIMER]


class Mode(_ClimateFunit):
    entity_type = Thermoregulator

    MANUAL = 'Manual'
    AUTO = 'Auto'
    COOLING = 'Cooling'
    HEATING = 'Heating'
    DRYING = 'Drying'
    VENTILATION = 'Ventilation'
    HEATPLUS = 'HeatPlus'
    STOP = 'Stop'

    variants = [MANUAL, AUTO, COOLING, HEATING, DRYING, VENTILATION, HEATPLUS, STOP]


class FanSpeedMode(_ClimateFunit):
    entity_type = Thermoregulator

    MANUAL = 'Manual'
    AUTO = 'Auto'
    HIGH = 'High'
    MEDIUM = 'Medium'
    LOW = 'Low'
    TOP = 'Top'

    variants = [MANUAL, AUTO, HIGH, MEDIUM, LOW, TOP]


class LouverMode(_ClimateFunit):
    entity_type = Thermoregulator

    MANUAL = 'Manual'
    AUTO = 'Auto'
    HORIZONTAL = 'Horizontal'
    ANGLE30 = 'Angle30'
    ANGLE45 = 'Angle45'
    ANGLE60 = 'Angle60'
    VERTICAL = 'Vertical'

    variants = [MANUAL, AUTO, HORIZONTAL, ANGLE30, ANGLE45, ANGLE60, VERTICAL]


Thermoregulator.settable_units = [
    On,
    TargetTemperature,
    Preset,
    Mode,
    FanSpeedMode,
    LouverMode
]

Fan.settable_units = [On]

HeatedFloor.settable_units = [On]
