import spread_core.bam.const as BAM
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from .enginery import Enginery, _EngineryFunit
from ..recipe import Recipe


class Shade(Enginery):
    pass


class Shutter(Shade):
    pass


class Curtain(Shade):
    pass


class Blind(Shade):
    POSITION_TYPE = 'Closeness'
    ANGLE_TYPE = 'Angle'

    def __init__(self, _id: int, **kwargs):
        super(Blind, self).__init__(_id, **kwargs)
        self._angle = None

    def set_recipe(self, ingredient_type, recipe: Recipe):
        if ingredient_type == self.POSITION_TYPE:
            self._recipe = recipe
        elif ingredient_type == self.ANGLE_TYPE:
            self._angle = recipe
        else:
            raise ValueError('unknown ingredient type: "%s"' % ingredient_type)

    def recipe_for_funit(self, funit):
        if funit == PositionAngle:
            return PositionLevel, self._angle
        elif funit == Rotation:
            return Motion, self._angle
        else:
            return super(Blind, self).recipe_for_funit(funit)

    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.id == self._angle.id:
            if topic.entity_address.funit_type == PositionLevel:
                self.publish_state(PositionAngle, None if variable.invalid else PositionAngle.level2angle(variable.value))
            elif topic.entity_address.funit_type == Motion:
                self.publish_state(Rotation, None if variable.invalid else Rotation.from_motion(variable.value))
        else:
            super(Blind, self).on_state(topic, variable)


class _ShadeFunit(_EngineryFunit):
    entity_type = Shade


Shade.funit_class = _ShadeFunit


class Motion(_ShadeFunit):
    variants = (BAM.MOTOR_OPEN, BAM.MOTOR_CLOSE, BAM.MOTOR_STOP)


class PositionLevel(_ShadeFunit):
    value_range = range(0, 10_001)


class Rotation(_ShadeFunit):
    entity_type = Shutter

    CW = 'CW'
    CCW = 'CCW'

    variants = (CW, CCW, BAM.MOTOR_STOP)
    state_variants = (CW, CCW, BAM.MOTOR_STOPPED)

    def set(self):
        if self.set_value == BAM.MOTOR_STOP:
            if self.entity.get_value(self.__class__) == BAM.MOTOR_STOPPED:
                raise AssertionError("Already stopped")
            else:
                state = BAM.MOTOR_STOPPED
        else:
            state = self._variable.value

        super(Rotation, self).set()

        self.entity.publish_state(Rotation, state)

    @property
    def set_value(self):
        rotation = super(Rotation, self).set_value

        if rotation == self.CW:
            return BAM.MOTOR_OPEN
        elif rotation == self.CCW:
            return BAM.MOTOR_CLOSE
        else:
            return BAM.MOTOR_STOP

    @classmethod
    def from_motion(cls, motion):
        if motion == BAM.MOTOR_OPENING:
            return cls.CW
        elif motion == BAM.MOTOR_CLOSING:
            return cls.CCW
        elif motion == BAM.MOTOR_STOPPED:
            return BAM.MOTOR_STOPPED
        else:
            return None


class PositionAngle(_ShadeFunit):
    entity_type = Shutter
    value_range = range(-9_000, 9_001)

    def set(self):
        has_angle = self.entity.get_value(PositionAngle)
        super(PositionAngle, self).set()

        if has_angle is not None:
            if self.set_value is None:
                pass
            elif self.set_value > has_angle:
                self.entity.publish_state(Rotation, Rotation.CCW)
            elif self.set_value < has_angle:
                self.entity.publish_state(Rotation, Rotation.CW)

    @property
    def set_value(self):
        return self.angle2level(super(PositionAngle, self).set_value)

    @staticmethod
    def level2angle(raw):
        return round(18_000 * raw / 10_000) - 9_000

    @staticmethod
    def angle2level(angle):
        if angle is not None:
            return round(10_000 * (angle + 9_000) / 18_000)

        return angle


Shade.settable_units = (
    Motion,
    PositionLevel,
)

Blind.settable_units = Shade.settable_units + (
    Rotation,
    PositionAngle,
)
