from spread_core.bam.engineries import alarm
from .subginery import Subginery, _SubgineriesFunit
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable


class Alarm(Subginery):
    def on_level_changed(self, eng, eng_funit_type=None):
        value = None
        state_changed = False
        invalid = False
        inconsistent = False

        if eng_funit_type == alarm.Guard:
            if isinstance(eng, alarm.IntruderSensor):
                funit = IntruderSensorsGuard

            elif isinstance(eng, alarm.FireSensor):
                funit = FireSensorsGuard

            elif isinstance(eng, alarm.LeakageSensor):
                funit = LeakageSensorsGuard

            else:
                return

        elif eng_funit_type == alarm.Triggered:
            if isinstance(eng, alarm.IntruderSensor):
                funit = IntruderSensorsTriggered

            elif isinstance(eng, alarm.FireSensor):
                funit = FireSensorsTriggered

            elif  isinstance(eng, alarm.LeakageSensor):
                funit = LeakageSensorsTriggered

            else:
                return

        else:
            return

        funit = str(funit)
        for current_eng in self.engineries:
            if isinstance(current_eng, eng.__class__):
                if eng_funit_type == alarm.Guard:
                    _value = current_eng.recipe.guarded
                else:
                    _value = current_eng.values.get(eng_funit_type)

                if _value is None:
                    invalid = True

                elif value is None:
                    value = _value

                elif value != _value:
                    inconsistent = True

                if invalid and inconsistent:
                    break

        if inconsistent:
            if eng_funit_type == alarm.Guard:
                value = None
            else:
                value = True

        if invalid:
            if funit not in self._invalid:
                self._invalid.append(funit)
                state_changed = True
        else:
            if funit in self._invalid:
                self._invalid.remove(funit)
                state_changed = True

        if inconsistent:
            if funit not in self._inconsistent:
                self._inconsistent.append(funit)
                state_changed = True
        else:
            if funit in self._inconsistent:
                self._inconsistent.remove(funit)
                state_changed = True

        if self.get_value(funit) != value:
            self.set_value(funit, value)
            state_changed = True

        if state_changed:
            self.adapter.publish(State(self.get_address(funit)), Variable(value, invalid=invalid, inconsistent=inconsistent, subginery=True), retain=True, delay=0)


class _AlarmFunit(_SubgineriesFunit):
    entity_type = Alarm
    target_class = alarm._AlarmSensor
    target_command = alarm.Guard


Alarm.funit_class = _AlarmFunit


class IntruderSensorsGuard(_AlarmFunit):
    target_class = alarm.IntruderSensor


class FireSensorsGuard(_AlarmFunit):
    target_class = alarm.FireSensor


class LeakageSensorsGuard(_AlarmFunit):
    target_class = alarm.LeakageSensor


class IntruderSensorsTriggered(_AlarmFunit):
    target_class = alarm.IntruderSensor

    def set(self):
        raise self.NotImplemented()


class FireSensorsTriggered(_AlarmFunit):
    target_class = alarm.FireSensor

    def set(self):
        raise self.NotImplemented()


class LeakageSensorsTriggered(_AlarmFunit):
    target_class = alarm.LeakageSensor

    def set(self):
        raise self.NotImplemented()


Alarm.settable_units = [IntruderSensorsGuard, FireSensorsGuard, LeakageSensorsGuard]
