# import json
import re
from spread_core.bam.validation.base import corr_req_par
from spread_core.bam.validation.enginery import EngineryValidator
import spread_core.bam.validation.constraints as constr


class SubgineryValidator:
    def __init__(self, data):
        if not isinstance(data, dict):
            raise ValueError('Wrong data structure (dict needed)')
        if not corr_req_par('type', str, data) or data['type'] not in constr.SUBG_CONSTR:
            raise ValueError('Wrong type')
        if not corr_req_par('id', int, data) or data['id'] < 1:
            raise ValueError('Wrong id')
        # key must be GUID
        if not corr_req_par('key', str, data) \
                or not re.match(r'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$', data['key']):
            raise ValueError('Wrong key')
        if not corr_req_par('locationID', int, data) or data['locationID'] < 1:
            raise ValueError('Wrong locationID')
        if 'engineries' in data:
            if not isinstance(data['engineries'], list):
                raise ValueError('Wrong engineries structure (list needed)')
            for eng in data['engineries']:
                if not isinstance(eng, int) or eng < 1:
                    raise ValueError('Wrong engineries')
        self.Attributes(data)

    def Attributes(self, data):
        error = f"Wrong {data['type']} attribute"
        if data['type'] == 'Climate':
            if 'attributes' not in data:
                raise ValueError(f"{error}s")
            if 'temperatureSensor' not in data['attributes']:
                raise ValueError(f"{error} temperatureSensor")
            if not corr_req_par('presence', bool, data['attributes']['temperatureSensor']):
                raise ValueError(f"{error} temperatureSensor presence")

            if 'thermoregulator' not in data['attributes']:
                raise ValueError(f"{error} thermoregulator")

            if not corr_req_par('presence', bool, data['attributes']['thermoregulator']):
                raise ValueError(f"{error} thermoregulator presence")

            # Проверка attributes объекта Thermoregulator через Engineries
            try:
                EngineryValidator.Attributes('Thermoregulator', data['attributes']['thermoregulator'])
            except ValueError as ex:
                raise ValueError(f"{error} thermoregulator ({ex})")

            if 'fan' not in data['attributes']:
                raise ValueError(f"{error} fan")
            if not corr_req_par('presence', bool, data['attributes']['fan']):
                raise ValueError(f"{error} fan presence")

            if 'heatedFloor' not in data['attributes']:
                raise ValueError(f"{error} heatedFloor")
            if not corr_req_par('presence', bool, data['attributes']['heatedFloor']):
                raise ValueError(f"{error} heatedFloor presence")

        elif 'attributes' in data:
            raise ValueError(f"{error}s")
