import unittest

import spread_core.bam.managers as m_lib
import spread_knx as knx
from spread_core.bam.knx.providers import KnxProvider
from spread_core.tests.hardware.utils import MockKnx, MockBus
from spread_core.tools.signature import CommandSignature


class TestKnxProvider(unittest.TestCase):
    Provider = KnxProvider
    address = knx.address.Address(state='0/0/1', control='0/1/0')

    @classmethod
    def setUpClass(cls) -> None:
        import logging
        logging.getLogger().setLevel(logging.WARNING)

        m_lib.config.args = [None]

    def setUp(self) -> None:
        self.bus = MockBus.of_knx()
        self.manager = MockKnx(self.bus)

        self.setUpProvider()

    def setUpProvider(self):
        provider = self.Provider(999, self.address)
        provider.set_manager(self.manager)
        self._provider = provider

    @property
    def provider(self) -> KnxProvider:
        return self._provider

    def _make_update(self, funit, value, session=None, key=None, sig=None):
        if sig is None:
            if session:
                sig = CommandSignature(funit, key=key, session_id=session)
            else:
                sig = None

        self.provider.on_update(funit, value, sig=sig)

    def _check_value(self, funit, expected_value):
        self.assertEqual(expected_value, self.provider.values[funit])

    def _check_state(self, funit, value, check_value=True, **attrs):
        if check_value:
            self._check_value(funit, value)
        self.manager.check_publish(self, f'Spread/State/1/Hardware/MockKnx/2/{self.provider.__class__.__name__}/999/{funit}', value, attrs=attrs)

    def _check_reply(self, session, key, value):
        self.manager.check_publish(self, f'Spread/Reply/1/{session}', value, _retain=False, attrs={'key': key})

    def _check_end_of_publishes(self):
        self.manager.check_end_of_publishes(self)

    def set_value(self, funit, value):
        if value is None:
            self.provider.values.pop(funit, None)
        else:
            self.provider.values[funit] = value


class TestUpdateHandling(TestKnxProvider):
    def test_easy(self):
        self._make_update('SomeUnit', 852)
        self._check_state('SomeUnit', 852)

        self._make_update('SomeUnit', 852)
        self._check_end_of_publishes()
