from unittest import TestCase

from spread_core.bam.dali import bindings
from spread_dali.address import Short, Group, Broadcast

of = bindings.of
BindingError = bindings.BindingError


class TestDaliBindings(TestCase):
    def test_missing_type(self):
        with self.assertRaises(BindingError) as ex:
            of({})

        self.assertEqual(ex.exception.code, BindingError.MISSING_KEY)

    def test_missing_device(self):
        with self.assertRaises(BindingError) as ex:
            of({'binding': "Device"})

        self.assertEqual(ex.exception.code, BindingError.MISSING_KEY)

    def test_missing_group(self):
        with self.assertRaises(BindingError) as ex:
            of({'binding': "Group"})

        self.assertEqual(ex.exception.code, BindingError.MISSING_KEY)

    def test_device(self):
        binding = of({
            'binding': "Device",
            'device': 63
        })

        self.assertIsInstance(binding, Short)
        self.assertEqual(binding.address, 63)

    def test_device_invalid(self):
        with self.assertRaises(BindingError) as ex:
            of({
                'binding': "Device",
                'device': 64
            })

        self.assertEqual(ex.exception.code, BindingError.INVALID_VALUE)

    def test_group(self):
        binding = of({
            'binding': 'Group',
            'group': 31
        })

        self.assertIsInstance(binding, Group)
        self.assertEqual(binding.group, 31)

    def test_group_invalid(self):
        with self.assertRaises(BindingError) as ex:
            of({
                'binding': 'Group',
                'group': 32
            })

        self.assertEqual(ex.exception.code, BindingError.INVALID_VALUE)

    def test_broadcast(self):
        binding = of({
            'binding': 'Broadcast'
        })

        self.assertIsInstance(binding, Broadcast)

    def test_unknown_binding(self):
        with self.assertRaises(BindingError) as ex:
            of({
                'binding': 'Qwerty'
            })

        self.assertEqual(ex.exception.code, BindingError.INVALID_BINDING)
