import unittest
# from unittest import TestCase
from spread_core.bam.validation.user import UserValidator


USER = {
    "id": 1,
    "key": "0a915318-1c29-46a0-a6a7-9373300f4551",
    "login": "\u041e\u0444\u0438\u0441\u0020\u0056\u0061\u0072\u0074\u006f\u006e",
    "passwordEncoding": "Sha512",
    "password": "EBBCFECBB97654304F3AEE0228236EC4D94FA0593EA1EC8236FC3E2B9E6B01546D9EE62550D8FF30D3DF9EDC65517CAA75DCB6DD358E11A9EC5D7CDF8911B0C0",
    "rootLocationID": 49028,
    "homeLocationID": 49028
}

USER_REQ_PARAM = {
    "id": True,
    "key": True,
    "login": True,
    "passwordEncoding": True,
    "password": True,
    "rootLocationID": True,
    "homeLocationID": False
}

class TestUserValidator(unittest.TestCase):
    template = {}

    def test_user(self):
        UserValidator(USER)

    def test_require_param_negative(self):
        for par_name in USER:
            with self.subTest(parameter=par_name):
                TEST_USER = USER.copy()
                del TEST_USER[par_name]

                if USER_REQ_PARAM[par_name]:
                    with self.assertRaises(ValueError):
                        UserValidator(TEST_USER)
                else:
                    UserValidator(TEST_USER)

    def test_id_negative(self):
        TEST_USER = USER.copy()
        for id in -1, 0, '1', [1], ('1',), {}, None, True:
            with self.subTest(user_id=id):
                TEST_USER['id'] = id
                with self.assertRaises(ValueError):
                    UserValidator(TEST_USER)

    def test_key_negative(self):
        TEST_USER = USER.copy()
        for key in 1, '1', [1], None, '0a915318-1c29-46a0-a6a7-9373300f455g':
            with self.subTest(user_key=key):
                TEST_USER['key'] = key
                with self.assertRaises(ValueError):
                    UserValidator(TEST_USER)

    def test_login_negative(self):
        TEST_USER = USER.copy()
        for login in 1, [1], None:
            with self.subTest(user_login=login):
                TEST_USER['login'] = login
                with self.assertRaises(ValueError):
                    UserValidator(TEST_USER)

if __name__ == '__main__':
    unittest.main()
