from spread_core.mqtt import jocket, TopicError
from spread_core.tests.topics.test_common import TestTopic


class TestJocketAddress(TestTopic):
    def parse_invalid_address(self, address):
        with self.assertRaises(TopicError):
            self.parse_address(address)

    @staticmethod
    def parse_address(address):
        return jocket.address.EntityAddress.deserialize(*address.split('/'))

    def test_invalid(self):
        self.parse_invalid_address("Hardware")
        self.parse_invalid_address("Hardware/Server")
        self.parse_invalid_address("Hardware/Server/2")
        self.parse_invalid_address("Hardware/Server/2/Manager")
        self.parse_invalid_address("Hardware/Server/2/Manager/3")

        self.parse_invalid_address("Hardware/Server/2/Manager/3/Provider/4")
        self.parse_invalid_address("Hardware/Server/2/Manager/3/Provider/4/Funit/qwe")

        self.parse_invalid_address("Equipment")
        self.parse_invalid_address("Equipment/Enginery")
        self.parse_invalid_address("Equipment/Enginery/2")
        self.parse_invalid_address("Equipment/Enginery/2/Funit/3")

    def test_manager_address(self):
        self.assertEqual(
            jocket.address.ManagerAddress("Server", 2, "Manager", 3, "Funit"),
            self.parse_address("Hardware/Server/2/Manager/3/Funit")
        )

    def test_provider_address(self):
        self.assertEqual(
            jocket.address.ProviderAddress("Server", 2, "Manager", 3, "Provider", 4, "Funit"),
            self.parse_address('Hardware/Server/2/Manager/3/Provider/4/Funit')
        )

    def test_enginery_address(self):
        self.assertEqual(
            jocket.address.EngineryAddress("Enginery", 2, "Funit"),
            self.parse_address('Equipment/Enginery/2/Funit')
        )

    def test_subginery_address(self):
        self.assertEqual(
            jocket.address.SubgineryAddress("Subginery", 2, "Funit"),
            self.parse_address('Equipment/Subginery/2/Funit')
        )


class TestJocketTopic(TestTopic):
    def test_invalid(self):
        self.parse_invalid_topic("Jocket")

        self.parse_invalid_topic("Jocket/State")
        self.parse_invalid_topic("Jocket/State/1")

        self.parse_invalid_topic("Jocket/Reply")
        self.parse_invalid_topic("Jocket/Reply/1/Session/qwe")

        self.parse_invalid_topic("Jocket/Command")
        self.parse_invalid_topic("Jocket/Command/1")

    def test_state(self):
        self.assertEqual(
            jocket.topic.TopicState(1, jocket.address.EngineryAddress("Enginery", 2, "Funit")),
            self.PARSE("Jocket/State/1/Equipment/Enginery/2/Funit")
        )

    def test_command(self):
        self.assertEqual(
            jocket.topic.TopicCommand(1, "Session", jocket.address.EngineryAddress("Enginery", 2, "Funit")),
            self.PARSE("Jocket/Command/1/Session/Equipment/Enginery/2/Funit")
        )

    def test_reply(self):
        self.assertEqual(
            jocket.topic.TopicReply(1, "Session"),
            self.PARSE("Jocket/Reply/1/Session")
        )
