class HardwareWarehouse:
    __slots__ = ['providers', 'bindings', 'groups']

    def __init__(self):
        self.providers = {}
        self.bindings = {}
        self.groups = {}

    def all(self):
        yield from self.providers.values()

    def add_provider(self, provider):
        self.providers[provider.id] = provider

    def add_binding(self, binding, provider):
        if binding not in self.bindings:
            self.bindings[binding] = set()

        self.bindings[binding].add(provider)

    def sorted_info(self) -> ():
        yield from sorted(self.providers.values(), key=lambda _p: _p.rank)

    def get_provider(self, provider_id):
        """returns provider by id"""
        return self.providers.get(provider_id)

    def providers_by_address(self, address):
        """returns providers by address"""

        if address in self.bindings:
            yield from self.bindings[address]

    def include_in_group(self, group_id, participant):
        if group_id not in self.groups:
            self.groups[group_id] = set()

        self.groups[group_id].add(participant)

    def exclude_from_group(self, group_id, participant):
        if group_id in self.groups:
            self.groups[group_id].discard(participant)
            if self.group_size(group_id) == 0:
                del self.groups[group_id]

    def group_participants(self, group_id):
        if group_id in self.groups:
            yield from self.groups[group_id]

    def group_size(self, group_id):
        if group_id in self.groups:
            return len(self.groups[group_id])

        return 0
