from django.contrib.auth.decorators import login_required
from django.shortcuts import render

from spread_core import mqtt
from spread_core.bam.subgineries.incubator import Incubator
from spread_core.tools.settings_const import CYCLES_FROM_0
from spread_dashboard.project import bam
from spread_dashboard.utils import app


@login_required
def locations(request):
    return render(
        request, 'incubator/locations.html',
        dict(
            broker=bam.broker_auth(),
            start_from_0=app().settings.get(CYCLES_FROM_0, False),
            locations=mqtt.project.ProjectFile.locations(app().project_id),
            cycles=mqtt.project.Cycle.broadcast(app().project_id, ''),
            states=mqtt.spread.topic.State(
                mqtt.spread.address.SubgineryAddress.broadcast(app().project_id, subginery_type=Incubator.__name__)
            ),
            start_index=0 if app().settings.get(CYCLES_FROM_0, False) else 1
        )
    )


@login_required
def cycles(request):
    return render(request, 'incubator/cycles.html', dict(
        broker=bam.broker_auth(),
        cycles=mqtt.project.Cycle.broadcast(app().project_id, ''),
        triggers=mqtt.project.CycleTrigger.broadcast(app().project_id),
        local=mqtt.project.LOCAL,
        set=mqtt.project.SET,
        reset=mqtt.project.RESET,
        start_index=0 if app().settings.get(CYCLES_FROM_0, False) else 1
    ))


@login_required
def cycle(request, cid):
    return render(request, 'incubator/cycle.html', dict(
        start_index=0 if app().settings.get(CYCLES_FROM_0, False) else 1,
        broker=bam.broker_auth(),
        data=str(mqtt.project.Cycle(app().project_id, cid)),
        triggers=str(mqtt.project.CycleTrigger.broadcast(app().project_id, cid, '')),
        local=mqtt.project.LOCAL,
        set=mqtt.project.SET,
        reset=mqtt.project.RESET,
    ))
