from unittest import skip

from django.test import TestCase
from django.core.management import call_command
from django.urls import resolve
from django.views.static import serve as media_serve

from spread_dashboard.commissioning import views, views_editor, views_print


EDIT_BUTTON = 'id="edit-button"'
PRINT_LOCATION_BUTTON = 'id="print-location-button"'
PRINT_BUTTON = 'id="print-button"'
DATE_PICKER = 'id="date-picker"'

class ViewTestCase(TestCase):

    # @classmethod
    # def setUpClass(cls):
    #     call_command('loaddata', 'fixture.json')
    #
    # @classmethod
    # def tearDownClass(cls):
    #     pass

    @skip('TODO разобраться с редиректом и что ассертить')
    def test_deny_anonymous(self):
        # call_command('loaddata', 'fixture.json')
        res = self.client.get('/spread/')
        self.assertRedirects(res, '/accounts/login/')

    def test_commissioning_login_view(self):
        # call_command('loaddata', 'fixture.json')
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/')
        self.assertEqual(res.status_code, 200)

    def test_commissioning_root_url_resolves_to_index(self):
        res = resolve('/spread/commissioning/')
        self.assertEqual(res.func.view_class, views.CommissioningIndex)

    def test_commissioning_location_url_resolves(self):
        res = resolve('/spread/commissioning/123/')
        self.assertEqual(res.func.view_class, views.CommissioningLocation)

    def test_commissioning_area_url_resolves(self):
        res = resolve('/spread/commissioning/123/456/')
        self.assertEqual(res.func.view_class, views.CommissioningArea)

    def test_commsissioning_edit_location_url_resolves(self):
        res = resolve('/spread/commissioning/admin/123/')
        self.assertEqual(res.func, views_editor.of_location)

    def test_commissioning_edit_area_url_resolves(self):
        res = resolve('/spread/commissioning/admin/123/456/')
        self.assertEqual(res.func, views_editor.of_area)

    def test_commissioning_print_url_resolves(self):
        res = resolve('/spread/commissioning/print')
        self.assertEqual(res.func, views_print.for_print)

    def test_commissioning_intro_url_resolves(self):
        res = resolve('/spread/commissioning/intro')
        self.assertEqual(res.func.view_class, views.Intro)

    def test_commissioning_glossary_url_resolves(self):
        res = resolve('/spread/commissioning/glossary')
        self.assertEqual(res.func.view_class, views.Glossary)

    def test_media_url_resolves(self):
        res = resolve('/spread/media/123/media/123_1.jpg')
        self.assertEqual(res.func, media_serve)

    def test_index_page(self):
        call_command('loaddata', 'fixture.json')
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/commissioning/')
        html = res.content.decode('utf-8')
        self._check_absence_of_edit_print_button_and_datepicker(html)
        self.assertIn(PRINT_BUTTON, html)

    def test_intro_page(self):
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/commissioning/intro')
        html = res.content.decode('utf-8')
        self._check_absence_of_edit_print_button_and_datepicker(html)
        self.assertIn(PRINT_BUTTON, html)

    def test_glossary_page(self):
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/commissioning/glossary')
        html = res.content.decode('utf-8')
        self._check_absence_of_edit_print_button_and_datepicker(html)
        self.assertIn(PRINT_BUTTON, html)

    def test_location_page(self):
        call_command('loaddata', 'fixture.json')
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/commissioning/17099/')
        html = res.content.decode('utf-8')
        self._check_presense_of_edit_print_button_and_datepicker(html)
        self.assertIn(PRINT_BUTTON, html)

    def test_area_page(self):
        call_command('loaddata', 'fixture.json')
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/commissioning/17099/395439/')
        html = res.content.decode('utf-8')
        self._check_presense_of_edit_print_button_and_datepicker(html)
        self.assertIn(PRINT_BUTTON, html)

    def test_location_edit_page(self):
        call_command('loaddata', 'fixture.json')
        self.client.login(username='admin', password='proadmin')
        res = self.client.get('/spread/commissioning/admin/17099/')
        html = res.content.decode('utf-8')
        self._check_absence_of_edit_print_button_and_datepicker(html)
        self.assertNotIn(PRINT_BUTTON, html)

    @skip('TODO разрулить с app.settings в SchBase.commit_timer()')
    def test_area_edit_page(self):
        call_command('loaddata', 'fixture.json')
        self.client.login(username='admin', password='proadmin')
        #TODO разрулить с app.settings в SchBase.commit_timer()
        res = self.client.get('/spread/commissioning/admin/17099/395439/')
        html = res.content.decode('utf-8')
        self._check_absence_of_edit_print_button_and_datepicker(html)
        self.assertNotIn(PRINT_BUTTON, html)

    def _check_absence_of_edit_print_button_and_datepicker(self, html):
        # нет кнопки Редактировать
        self.assertNotIn(EDIT_BUTTON, html)
        # нет кнопки Отчет по локации
        self.assertNotIn(PRINT_LOCATION_BUTTON, html)
        # нет выбора дат
        self.assertNotIn(DATE_PICKER, html)

    def _check_presense_of_edit_print_button_and_datepicker(self, html):
        # есть кнопка Редактировать
        self.assertIn(EDIT_BUTTON, html)
        # есть кнопка Отчет по локации
        self.assertIn(PRINT_LOCATION_BUTTON, html)
        # есть выбор дат
        self.assertIn(DATE_PICKER, html)
